
package cn.ibizlab.api.rest.extensions;

import cn.ibizlab.core.uaa.extensions.service.UAACoreService;
import cn.ibizlab.util.client.IBZOUFeignClient;
import cn.ibizlab.util.security.AuthTokenUtil;
import cn.ibizlab.util.security.AuthenticationInfo;
import cn.ibizlab.util.security.AuthenticationUser;
import cn.ibizlab.util.security.AuthorizationLogin;
import cn.ibizlab.util.service.AuthenticationUserService;
import com.alibaba.fastjson.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.ResponseEntity;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.authority.AuthorityUtils;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import java.util.*;

/**
 * 客户端登录认证
 */
@RestController
@RequestMapping("/")
public class ClientAuthenticationResource
{

    @Value("${ibiz.jwt.header:Authorization}")
    private String tokenHeader;

    @Autowired
    private AuthTokenUtil jwtTokenUtil;

    @Autowired
    @Qualifier("IBZUSERService")
    private AuthenticationUserService userDetailsService;

    @Autowired
    private IBZOUFeignClient ouFeignClient;

    @PostMapping(value = "v7/login")
    public ResponseEntity<AuthenticationInfo> login(@Validated @RequestBody AuthorizationLogin authorizationLogin){
        userDetailsService.resetByUsername(authorizationLogin.getUsername());
        AuthenticationUser user = userDetailsService.loadUserByLogin(authorizationLogin.getUsername(),authorizationLogin.getPassword());
        if(user.getPermissionList()==null)
        {
            setUserPermission(user);
            setUserOrgInfo(user);
        }
        final String token = jwtTokenUtil.generateToken(user);
        // 返回 token
        return ResponseEntity.ok().body(new AuthenticationInfo(token,user));
    }

    @PostMapping(value = "uaa/login")
    public ResponseEntity<AuthenticationUser> uaalogin(@Validated @RequestBody AuthorizationLogin authorizationLogin){
        userDetailsService.resetByUsername(authorizationLogin.getUsername());
        AuthenticationUser user = userDetailsService.loadUserByLogin(authorizationLogin.getUsername(),authorizationLogin.getPassword());
        if(user.getPermissionList()==null)
        {
            setUserPermission(user);
            setUserOrgInfo(user);
        }
        return ResponseEntity.ok().body(user);
    }


    @PostMapping(value = "uaa/loginbyusername")
    public ResponseEntity<AuthenticationUser> loginByUsername(@Validated @RequestBody String username){
        //userDetailsService.resetByUsername(authorizationLogin.getUsername());
        AuthenticationUser user = userDetailsService.loadUserByUsername(username);
        if(user.getPermissionList()==null)
        {
            setUserPermission(user);
            setUserOrgInfo(user);
        }
        return ResponseEntity.ok().body(user);
    }

    @Autowired
    UAACoreService uaaCoreService;


    /**
     * 设置用户权限
     * 由于GrantedAuthority缺少无参构造，导致无法序列化，暂时通过PermissionList中转
     * @param user
     * @return
     */
    public void setUserPermission(AuthenticationUser user) {
        Collection<GrantedAuthority> userAuthorities=uaaCoreService.getAuthoritiesByUserId(user.getUserid());
        Set<String> authorities = AuthorityUtils.authorityListToSet(userAuthorities);
        if(user.getSuperuser()==1){
            authorities.add("ROLE_SUPERADMIN");
        }
        JSONObject permission =new JSONObject();
        permission.put("authorities",authorities);
        user.setPermissionList(permission);
    }

    /**
     * 设置用户组织相关信息
     * @param user
     */
    private void setUserOrgInfo(AuthenticationUser user) {

        Map<String, Set<String>> orgInfo=ouFeignClient.getOUMapsByUserId(user.getUserid());
        if(orgInfo==null)
            orgInfo=new HashMap<>();
            //throw new RuntimeException(String.format("获取用户信息失败，请检查用户中心[IBZOU]中是否存在[%s]用户!",user.getLoginname()));

        user.setOrgInfo(orgInfo);
    }
}
